@@ -305,7 +305,7 @@ hangul_engine_shape (PangoEngineShape *engine,
 		     PangoFont        *font,
 		     const char       *text,
 		     gint              length,
-		     PangoAnalysis    *analysis,
+		     const PangoAnalysis *analysis,
 		     PangoGlyphString *glyphs)
 {
   int n_chars, n_glyphs;
