@@ -266,7 +266,8 @@ syriac_engine_shape (PangoEngineShape *engine,
 
   fc_font = PANGO_FC_FONT (font);
   face = pango_fc_font_lock_face (fc_font);
-  g_return_if_fail (face != NULL);
+  if (!face)
+    return;
 
   ruleset = get_ruleset (face);
   if (!ruleset)
