@@ -30,6 +30,8 @@
 #include "pango-fontmap.h"
 #include "pango-utils.h"
 
+static const char bad_font_warning[] = "%s called with bad font, expect ugly output";
+
 struct _PangoFontDescription
 {
   char *family_name;
@@ -358,7 +360,7 @@ pango_font_description_set_size (PangoFontDescription *desc,
  * @desc: a #PangoFontDescription
  * 
  * Gets the size field of a font description.
- * See pango_font_description_get_size().
+ * See pango_font_description_set_size().
  * 
  * Return value: the size field for the font description in points or device units.
  *   You must call pango_font_description_get_size_is_absolute()
@@ -798,7 +800,8 @@ find_field (const FieldMap *map, int n_elements, const char *str, int len, int *
   
   for (i=0; i<n_elements; i++)
     {
-      if (map[i].str && g_ascii_strncasecmp (map[i].str, str, len) == 0)
+      if (map[i].str && map[i].str[0] && g_ascii_strncasecmp (map[i].str, str, len) == 0 &&
+	  map[i].str[len] == '\0')
 	{
 	  if (val)
 	    *val = map[i].value;
@@ -1172,7 +1175,7 @@ pango_font_get_glyph_extents  (PangoFont      *font,
       if (!_pango_warning_history.get_glyph_extents)
         {
 	  _pango_warning_history.get_glyph_extents = TRUE;
-	  g_warning ("pango_font_get_glyph_extents called with bad font, expect ugly output");
+	  g_warning (bad_font_warning, "pango_font_get_glyph_extents");
 	}
       if (ink_rect)
         {
@@ -1213,6 +1216,17 @@ PangoFontMetrics *
 pango_font_get_metrics (PangoFont        *font,
 			PangoLanguage    *language)
 {
+  if (G_UNLIKELY (!PANGO_IS_FONT (font)))
+    {
+      
+      if (!_pango_warning_history.get_metrics)
+        {
+	  _pango_warning_history.get_metrics = TRUE;
+	  g_warning (bad_font_warning, "pango_font_get_metrics");
+	}
+      return pango_font_metrics_new ();
+    }
+
   return PANGO_FONT_GET_CLASS (font)->get_metrics (font, language);
 }
 
@@ -1229,6 +1243,17 @@ pango_font_get_metrics (PangoFont        *font,
 PangoFontMap *
 pango_font_get_font_map (PangoFont *font)
 {
+  if (G_UNLIKELY (!PANGO_IS_FONT (font)))
+    {
+      
+      if (!_pango_warning_history.get_font_map)
+        {
+	  _pango_warning_history.get_font_map = TRUE;
+	  g_warning (bad_font_warning, "pango_font_get_font_map");
+	}
+      return NULL;
+    }
+
   if (PANGO_FONT_GET_CLASS (font)->get_font_map)
     return PANGO_FONT_GET_CLASS (font)->get_font_map (font);
   else
